import { Signature, createVAA, deserialize, serialize } from "../../index.js";
import { keccak256, secp256k1 } from "../../utils.js";
export class MockGuardians {
    setIndex;
    signers;
    constructor(setIndex, keys) {
        this.setIndex = setIndex;
        this.signers = keys.map((key, index) => {
            return { index, key };
        });
    }
    getPublicKeys() {
        return this.signers.map((guardian) => ethPrivateToPublic(guardian.key));
    }
    addSignatures(message, guardianIndices) {
        if (guardianIndices.length == 0)
            throw Error("guardianIndices.length == 0");
        const signers = this.signers.filter((signer) => guardianIndices.includes(signer.index));
        const vaa = deserialize("Uint8Array", message);
        for (let i = 0; i < signers.length; ++i) {
            const signer = signers.at(i);
            if (!signer)
                throw Error("No signer with index: " + i);
            const signature = ethSignWithPrivate(signer.key, keccak256(vaa.hash));
            const s = new Signature(signature.r, signature.s, signature.recovery);
            // @ts-ignore -- wants it to be immutable
            vaa.signatures.push({ guardianIndex: i, signature: s });
        }
        return vaa;
    }
}
export class MockEmitter {
    chain;
    address;
    sequence;
    constructor(emitterAddress, chain, startSequence) {
        this.chain = chain;
        this.address = emitterAddress;
        this.sequence = startSequence == undefined ? 0n : startSequence;
    }
    publishMessage(nonce, payload, consistencyLevel, timestamp, uptickSequence = true) {
        if (uptickSequence) {
            ++this.sequence;
        }
        return serialize(createVAA("Uint8Array", {
            guardianSet: 0,
            signatures: [],
            nonce: nonce,
            timestamp: timestamp ?? 0,
            sequence: this.sequence,
            emitterChain: this.chain,
            emitterAddress: this.address,
            consistencyLevel: consistencyLevel,
            payload: payload,
        }));
    }
}
export function ethPrivateToPublic(privateKey) {
    return secp256k1.getPublicKey(privateKey);
}
export function ethSignWithPrivate(privateKey, hash) {
    if (hash.length != 32)
        throw new Error("hash.length != 32");
    return secp256k1.sign(hash, privateKey);
}
export function ethValidateSig(signature, publicKey, hash) {
    const { r, s } = signature;
    return secp256k1.verify({ r, s }, hash, publicKey);
}
//# sourceMappingURL=guardian.js.map